C     PROGRAM BERNO 
C
C     BERNOULLI'S METHOD
C 
      PARAMETER(ND=20)
      DIMENSION A(ND),Y(ND)
      DOUBLE PRECISION EPS,A,Y,X,PX
      COMPLEX*16 Z,PZ
      READ(5,*) N,NIT,EPS
      N1=N+1
      READ(5,*) (A(I),I=1,N1)
      READ(5,*) (Y(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' NIT=',NIT,' EPS=',EPS
      WRITE(6,*) 'A= ',(A(I),I=1,N1)
      WRITE(6,*) 'Y= ',(Y(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL BERNOUL(ND,N,NIT,EPS,A,Y,X,PX,Z,PZ,KIT,ITEST)
      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      WRITE(6,*) 'X=',X
      WRITE(6,*) 'PX=',PX
      WRITE(6,*) 'Z=',Z
      WRITE(6,*) 'PZ=',PZ
      STOP
      END
C-----------------------------------------------------------------
      SUBROUTINE BERNOUL(ND,N,NIT,EPS,A,Y,X,PX,Z,PZ,KIT,ITEST)
C
C     COMPUTATION OF THE LARGEST ROOT (IN MODULUS) OF A POLYNOMIAL
C     P BY BERNOULLI'S METHOD
C
C     INPUT:
C     ND: MAXIMUM DIMENSION OF ARRAYS
C     N: DEGREE OF POLYNOMIAL
C     NIT: MAXIMUM ITERATION INDEX (NIT>N)
C     EPS: CONVERGENCE TEST NUMBER
C     A: VECTOR OF POLYNOMIAL COEFFICIENTS (N+1) 
C     Y: VECTOR OF INITIAL VALUES (N)
C
C     OUTPUT:
C     X: APPROXIMATE LARGEST ROOT (REAL), IF ITEST=1
C     PX:=P(X)
C     Z: APPROXIMATE LARGEST ROOT (NONREAL), IF ITEST=2
C     PZ:=P(Z) 
C     KIT: ITERATION INDEX (KIT>N)       
C     ITEST = 1, CONVERGENCE TEST SATISFIED AND THE L. ROOT IS REAL
C           = 2, CONV. TEST SATISFIED AND THE L. ROOT IS NONREAL
C           = 3, CONV. TEST NOT SATISFIED
C
      DIMENSION A(ND),Y(ND)
      DOUBLE PRECISION EPS,A,Y,X,PX,S,Y1,D,DM,E,DD,R,DDM,C,U,V,EE
      COMPLEX*16 Z,PZ
      EE=1.D-12
C      
C     ITERATIONS
C
      N1=N+1
      DO 50 K=N1,NIT
      KIT=K
C
C     COMPUTATION OF Y(K)
C
      S=0.D0
         DO 60 L=1,N
         I=N1-L
         J=K-L
   60    S=S-A(I)*Y(J)
      Y(K)=S/A(N1)
      IF(K.LT.4) GO TO 50
C
C     COMPUTATION OF X AND  Z:= U + V I
C
      Y1=Y(K-1)
      IF(DABS(Y1).LT.EE) Y1=EE
      X=Y(K)/Y1
      D=Y1**2-Y(K)*Y(K-2)
      DM=Y(K-2)**2-Y1*Y(K-3)
      IF(DABS(DM).LE.EE) DM=EE
      E=Y1*Y(K-2)-Y(K)*Y(K-3)
      DD=D/DM
      IF(DD.LT.0.D0) DD=0.D0
      R=DSQRT(DD)
      DDM=D*DM
      IF(DDM.LE.0.D0) DDM=EE
      C=E/DSQRT(DDM)/2.D0
      U=R*C
      S=1.D0-C*C
      IF(S.LT.0.D0) S=0.D0
      V=R*DSQRT(S)
      Z=DCMPLX(U,V)
      WRITE(6,*) 'K=',K,'  X=',X
      WRITE(6,*) '  Z=',Z
C
C     EVALUATION OF THE POLYNOMIAL 
C
      PX=A(N1)
      PZ=PX
         DO 70 L=1,N
         I=N1-L
         PX=X*PX+A(I)
   70    PZ=Z*PZ+A(I)
      IF(DABS(PX).LE.EPS) GO TO 10
      IF(CDABS(PZ).LE.EPS) GO TO 20
   50 CONTINUE
      ITEST=3
      RETURN
   10 ITEST=1
      RETURN
   20 ITEST=2
      RETURN
      END
