C     PROGRAM CHOLE
C
C     CHOLESKI'S METHOD
C
      PARAMETER(ND=10)
      DOUBLE PRECISION A,B
      DIMENSION A(ND,ND),B(ND)
      READ(5,*) N,IOPT
      READ(5,*) ((A(I,J),J=1,I),I=1,N)
      READ(5,*) (B(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' IOPT=',IOPT
      WRITE(6,*) 'A='
      DO 10 I=1,N
      WRITE(6,*) (A(I,J),J=1,I)
   10 WRITE(6,*) ' '
      WRITE(6,*) 'B=',(B(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL CHOLES(ND,N,A,B,IOPT)
      WRITE(6,*) 'X=',(B(I),I=1,N)
      STOP
      END
C-----------------------------------------------------------------
      SUBROUTINE CHOLES(ND,N,A,B,IOPT)
C
C     DECOMPOSITION A=A(AT) OF THE MATRIX A AND/OR SOLUTION
C     OF THE LINEAR SYSTEM AX=B BY CHOLESKI'S METHOD.
C     A MUST BE SYMMETRIC AND POSITIVE DEFINITE
C
C     INPUT:
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     A: SYSTEM MATRIX, LOWER TRIANGULAR PART ONLY (NXN)
C     B: RIGHT-HAND SIDE VECTOR (N) 
C     IOPT = 0, DECOMPOSITION OF A AND SOLUTION OF THE SYSTEM 
C          = 1, DECOMPOSITION OF A ONLY
C          = 2, SOLUTION OF THE SYSTEM (SKIPS DECOMPOSITION)
C
C     OUTPUT:
C     A: DECOMPOSED MATRIX, LOWER PART, IF IOPT=0 OR 1 
C     B:=X: SOLUTION VECTOR STORED IN B (N)
C
      DOUBLE PRECISION A,B,S
      DIMENSION A(ND,N),B(ND)
      IF(IOPT.EQ.2) GO TO 45
C
C     DECOMPOSITION OF A=A(AT)
C
      DO 10 I=1,N
      IM=I-1
      IP=I+1
      S=0.D0
      IF(I.EQ.1) GO TO 15
         DO 20 K=1,IM
   20    S=S+A(I,K)**2
   15 A(I,I)=DSQRT(A(I,I)-S)
      IF(I.EQ.N) GO TO 10
         DO 30 J=IP,N
         S=0.D0
         IF(I.EQ.1) GO TO 30
            DO 40 K=1,IM
   40       S=S+A(I,K)*A(J,K)
   30    A(J,I)=(A(J,I)-S)/A(I,I)
   10 CONTINUE
      IF(IOPT.EQ.1) RETURN
C
C     SOLUTION OF THE SYSTEM
C
   45 B(1)=B(1)/A(1,1)
      DO 50 K=2,N
      KM=K-1
      S=0.D0
         DO 60 J=1,KM
   60    S=S+A(K,J)*B(J)
   50 B(K)=(B(K)-S)/A(K,K)
C   
      NM=N-1
      B(N)=B(N)/A(N,N)
      DO 80 I=1,NM
      K=N-I
      KP=K+1
      S=0.D0
         DO 90 J=KP,N
   90    S=S+A(J,K)*B(J)
   80 B(K)=(B(K)-S)/A(K,K)
      RETURN
      END

