      PROGRAM CONDE
C
C     ESTIMATION OF THE CONDITION NUMBER OF A MATRIX
C
      PARAMETER(ND=20)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(ND,ND),Y(ND),LPIV(ND)
      READ(5,*) N
C      READ(5,*) ((A(I,J),J=1,N),I=1,N)
C
C     HILBERT MATRIX (TEST)
      DO 1 I=1,N
      DO 1 J=1,N
    1 A(I,J)=1.D0/(I+J-1)
C
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N
      WRITE(6,*) 'A='
      DO 10 I=1,N
      WRITE(6,*) (A(I,J),J=1,N)
   10 WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
C
C     COMPUTE NORM1 OF A
C                                                                  
      AN=0.D0                                                   
      DO 20 J=1,N                                                  
      S=0.D0                                                       
      DO 30 I=1,N                                                   
   30 S=S+DABS(A(I,J))                                    
      IF(S.GT.AN) AN=S                                       
   20 CONTINUE                                                 
C
C     TRIANGULAR DECOMPOSITION OF A=L*U, WHERE L AND U ARE 'STORED' 
C     IN THE LOWER AND UPPER-TRIANGULAR PART OF A, BY GAUSS' METHOD
C
      CALL DGAUSS(ND,N,A,Y,1,D,IREG,LPIV)
C
      WRITE(6,*) 'D=',D,' IREG=',IREG
      IF(IREG.EQ.0) STOP
C
C     ESTIMATE THE CONDITION NUMBER OF A :
C     - SOLVE THE SYSTEM AT*Y=E BY SOLVING SUCCESSIVELY THE 
C       LOWER AND UPPER-TRIANGULAR SYSTEMS UT*Y'=E AND LT*Y=Y'.   
C       E IS A VECTOR WITH COMPONENTS +1 OR -1 CHOSEN SO AS TO
C       MAXIMIZE THE COMPONENTS OF Y' IN ABSOLUTE VALUE.
C     - SOLVE THE SYSTEM U*Z=Y.
C     - COMPUTE THE ESTIMATE
C          CONDEST=(NORM1 OF A)*(NORM1 OF Z)/(NORM1 OF Y).
C       
C     SOLVE THE SYSTEM UT*Y'=E BY FORWORD-SUBSTITUTION
C
      DO 50 K=1,N
      S=0.D0                                                        
      IF(K.EQ.1) GO TO 45                                           
      KM=K-1                                                       
         DO 40 J=1,KM                                                 
         S=S+A(J,K)*Y(J)                                            
   40    CONTINUE                                                       
   45 E=-1.D0                                                        
      IF(S.LT.0.D0) E=1.D0                                          
      Y(K)=(E-S)/A(K,K)                                          
   50 CONTINUE
C
C     SOLVE THE SYSTEM LT*Y=Y' BY BACK-SUBSTITUTION (THE DIAGONAL
C     ELEMENTS OF L ARE EQUAL TO 1) AND RE-ORDER THE COMPONENTS OF Y.
C
      DO 60 I=1,N-1                                               
      K=N-I                                                      
      S=0.D0                                                       
      KP=K+1                                                      
         DO 55 J=KP,N                                                
         S=S+A(J,K)*Y(J)                                            
   55    CONTINUE                                                
      Y(K)=Y(K)-S
C
      M=LPIV(K)
      IF(M.EQ.K) GO TO 60                                       
      S=Y(M)                                                    
      Y(M)=Y(K)
      Y(K)=S                                                    
   60 CONTINUE                                                  
C
C     COMPUTE NORM1 OF Y
C
      YN=0.D0
      DO 65 I=1,N                                               
      YN=YN+DABS(Y(I))                                    
   65 CONTINUE                                                  
C                                                               
C     SOLVE THE SYSTEM U*Z=Y BY BACH-SUBSTITUTION
C                                                                
      CALL DGAUSS(ND,N,A,Y,2,D,IREG,LPIV)
C
C     COMPUTE NORM1 OF Z
C
      ZN=0.D0                                                       
      DO 70 I=1,N                                                      
      ZN=ZN+DABS(Y(I))                                      
  70  CONTINUE                                                        
C                                                                      
C     COMPUTE THE ESTIMATE OF THE CONDITION NUMBER                                    
C                                                     
      CE=AN*ZN/YN
      IF(CE.LT.1.D0) CE=1.D0
      WRITE(6,*) 'CONDEST=',CE
      STOP
      END
C-------------------------------------------------------------------      
      SUBROUTINE DGAUSS(ND,N,A,B,IOPT,D,IREG,LPIV)
C       
C     DECOMPOSITION OF THE MATRIX A AND/OR SOLUTION OF THE
C     LINEAR SYSTEM  A*X=B  BY GAUSS' ELIMINATION METHOD
C
C     INPUT:
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     A: SYSTEM MATRIX (NxN)
C     B: RIGHT-HAND SIDE VECTOR (N)
C     IOPT = 0, DECOMPOSITION OF A AND SOLUTION OF THE SYSTEM
C          = 1, DECOMPOSITION OF A ONLY
C          = 2, SOLUTION OF THE SYSTEM (SKIPS DECOMPOSITION)
C        
C     OUTPUT:
C     A: DECOMPOSED MATRIX (LOWER\UPPER PART), IF IOPT=0 OR 1
C     B:=X: SOLUTION VECTOR STORED IN B (N)
C     D: DETERMINANT OF A                                                
C     IREG = 0, SINGULAR MATRIX A
C          = 1, REGULAR MATRIX A
C     LPIV: VECTOR OF PIVOTING INDICES (N)
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION A(ND,N),B(ND),LPIV(ND)
      EE=1.D-14
      IREG=1
      NM=N-1
      IF(IOPT.EQ.2) GO TO 60
C
C     TRIANGULAR DECOMPOSITION OF A WITH MAXIMUM COLUMN PIVOTING
C     (ELIMINATION WITH MULTIPLIERS STORED IN THE LOWER PART OF A)
C   
      D=1.D0
      DO 10 K=1,NM
      KP=K+1
      M=K
         DO 20 I=KP,N
         IF(DABS(A(I,K)).GT.DABS(A(M,K))) M=I
   20    CONTINUE
      LPIV(K)=M
      D=D*A(M,K)
      IF(DABS(A(M,K)).LE.EE) GO TO 100   
      IF(M.EQ.K) GO TO 30
      D=-D
         DO 40 J=K,N
         S=A(M,J)
         A(M,J)=A(K,J)
   40    A(K,J)=S
   30    DO 50 I=KP,N
         S=A(I,K)/A(K,K)
         A(I,K)=S
            DO 50 J=KP,N
   50       A(I,J)=A(I,J)-S*A(K,J)
   10 CONTINUE
      D=D*A(N,N)
      IF(DABS(A(N,N)).LE.EE) GO TO 100
      IF(IOPT.EQ.1) RETURN
C
C     ELIMINATION OPERATIONS ON B
C
   60 DO 70 K=1,NM
      KP=K+1
      M=LPIV(K)
      S=B(M)
      B(M)=B(K)
      B(K)=S
         DO 70 I=KP,N
   70    B(I)=B(I)-A(I,K)*S
C      
C     BACK-SUBSTITUTION
C
      B(N)=B(N)/A(N,N)
      DO 80 I=1,NM
      K=N-I
      KP=K+1
      S=0.D0
         DO 90 J=KP,N
   90    S=S+A(K,J)*B(J)
   80 B(K)=(B(K)-S)/A(K,K)
      RETURN
  100 IREG=0
      RETURN
      END
