C     PROGRAM CONGR
C
C     CONJUGATE GRADIENT METHOD
C      
      PARAMETER(ND=10)
      DOUBLE PRECISION EPS,A,B,X,G,D,AD,Q
      DIMENSION A(ND,ND),B(ND),X(ND),G(ND),D(ND),AD(ND)
      READ(5,*) N,NIT,EPS
      READ(5,*) ((A(I,J),J=1,I),I=1,N)
      READ(5,*) (B(I),I=1,N)
      READ(5,*) (X(I),I=1,N)
      WRITE(6,*) 'N=',N,' NIT=',NIT,' EPS=',EPS
      WRITE(6,*) 'A='
      DO 10 I=1,N
      WRITE(6,*) (A(I,J),J=1,I)
   10 WRITE(6,*) ' '
      WRITE(6,*) 'B=',(B(I),I=1,N)
      WRITE(6,*) 'X=',(X(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL CONGRAD(ND,N,A,B,X,NIT,EPS,Q,KIT,ITEST,G,D,AD)
      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      IF(ITEST.EQ.2) STOP
      WRITE(6,*) 'X=',(X(I),I=1,N)
      WRITE(6,*) 'Q=',Q
      STOP
      END
C---------------------------------------------------------------------
      SUBROUTINE CONGRAD(ND,N,A,B,X,NIT,EPS,Q,KIT,ITEST,G,D,AD)
C
C     MINIMIZATION OF THE QUADRATIC FUNCTION F(X)=(AX)*X/2-B*X,
C     OR SOLUTION OF THE LINEAR SYSTEM AX=B, BY THE CONJUGATE
C     GRADIENT METHOD. THE MATRIX A MUST BE SYMMETRIC AND
C     POSITIVE DEFINITE.
C
C     INPUT:
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     A: SYSTEM MATRIX, LOWER TRIANGULAR PART ONLY (NXN)
C     B: SYSTEM RIGHT-HAND SIDE VECTOR (N)
C     X: INITIAL VECTOR (N)
C     NIT: MAXIMUM NUMBER OF ITERATIONS
C     EPS: CONVERGENCE TEST NUMBER
C
C     OUTPUT:
C     X: SOLUTION VECTOR, IF ITEST=1 (N)
C     Q:=F(X): MINIMUM FUNCTION VALUE, IF ITEST=1
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     ITEST = 1, CONVERGENCE TEST SATISFIED
C           = 2, CONVERGENCE TEST NOT SATISFIED      
C     
      DOUBLE PRECISION EPS,A,B,X,G,D,AD,Q,R,S,T
      DIMENSION A(ND,N),B(ND),X(ND),G(ND),D(ND),AD(ND)
      DO 5 I=1,N
      DO 5 J=I,N
    5 A(I,J)=A(J,I)
C
      Q=0.D0
      DO 10 I=1,N
      S=0.D0
         DO 20 J=1,N
   20    S=S+A(I,J)*X(J)
      G(I)=S-B(I)
      Q=Q+G(I)**2
   10 D(I)=G(I)
C
      DO 30 K=0,NIT
      KIT=K 
      R=DSQRT(Q)
      IF(R.LE.EPS) GO TO 100
      R=0.D0
      T=0.D0
         DO 40 I=1,N
         S=0.D0
            DO 50 J=1,N
   50       S=S+A(I,J)*D(J)
         AD(I)=S
   40    T=T+S*D(I)                 
      R=Q/T 
      T=Q
      Q=0.D0
         DO 60 I=1,N 
         X(I)=X(I)-R*D(I)      
         G(I)=G(I)-R*AD(I)
   60    Q=Q+G(I)**2
      T=Q/T
         DO 70 I=1,N
   70    D(I)=G(I)+T*D(I)
      WRITE(6,*) 'X=',(X(I),I=1,N)
   30 CONTINUE
      ITEST=2
      RETURN 
C
C     MINIMUM FUNCTION VALUE
C
  100 ITEST=1    
      R=0.D0 
      T=0.D0
      DO 80 I=1,N
      S=0.D0
         DO 90 J=1,N
   90    S=S+A(I,J)*X(J)
      R=R+S*X(I)
   80 T=T+B(I)*X(I) 
      Q=R/2.D0-T
      RETURN
      END 



