C     PROGRAM FLETR 
C
C     FLETCHER-REEVES METHOD 
C
      PARAMETER(ND=10)
      DOUBLE PRECISION U,G,D,X,EPS,EP,SP,FU,FX,GN
      DIMENSION U(ND),G(ND),D(ND),X(ND)
      EXTERNAL GOLDSEC,XFX,GF
      READ(5,*) N,SP,NIT,EPS,NS,NT,EP
      READ(5,*) (U(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' SP= ',SP
      WRITE(6,*) 'NIT=',NIT,' EPS=',EPS
      WRITE(6,*) 'NS= ',NS,' NT= ',NT,' EP= ',EP
      WRITE(6,*) 'U=',(U(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL FLETRE(GOLDSEC,XFX,GF,ND,N,U,SP,NIT,EPS,
     2            NS,NT,EP,FU,X,FX,GN,KIT,ITEST,G,D)
      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      WRITE(6,*) 'X=',(X(I),I=1,N)
      WRITE(6,*) 'FX=',FX,' GN=',GN
      STOP
      END
C-------------------------------------------------------------------      
      SUBROUTINE FLETRE(GOLDSEC,XFX,GF,ND,N,U,SP,NIT,EPS,
     2                  NS,NT,EP,FU,X,FX,GN,KIT,ITEST,G,D)
C                               
C     MINIMIZATION OF THE FUNCTION F BY THE FLETCHER-REEVES METHOD 
C     (GENERALIZED CONJUGATE GRADIENT METHOD)
C
C     USES GOLDSEC, XFX AND GF SUBROUTINES
C
C     INPUT:
C     GOLDSEC: GOLDEN SECTION SEARCH METHOD SUBROUTINE
C     XFX: LINE X=U-RV AND FUNCTION FX=F(X) SUBROUTINE
C     GF: GRADIENT OF F SUBROUTINE
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     U: INITIAL VECTOR (N)
C     SP: STEP PARAMETER
C     NIT: MAXIMUM NUMBER OF ITERATIONS 
C     EPS: CONVERGENCE TEST NUMBER 
C     NS,NT,EP: SEE GOLDSEC SUBROUTINE 
C
C     OUTPUT:
C     X: LAST VECTOR ITERATE (APPROXIMATE SOLUTION) (N)
C     FX:=F(X)
C     U: BEFORE LAST VECTOR ITERATE (N)
C     FU:=F(U)
C     GN: NORM OF (GRAD F)(X)
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     ITEST = 1, CONVERGENCE TEST SATISFIED
C           = 2, CONVERGENCE TEST NOT SATISFIED
C
      DOUBLE PRECISION U,G,D,X,EPS,EP,SP,SK,GG,FU,FX,R,S,GN
      DIMENSION U(ND),G(ND),D(ND),X(ND)
      EXTERNAL XFX
      ITEST=1
      CALL GF(ND,N,U,G)
      CALL XFX(ND,N,U,G,0.D0,X,FX)
      FU=FX
      GG=0.D0
      DO 25 I=1,N
      GG=GG+G(I)**2
   25 D(I)=G(I)
      SK=SP
C
      DO 10 K=0,NIT
      KIT=K
      GN=DSQRT(GG)
      IF(GN.LE.EPS) RETURN
         DO 40 I=1,N
   40    U(I)=X(I)
C   
C     MINIMISATION OF F ON THE HALF-LINE U-RV, R>0, BY THE 
C     GOLDEN SECTION SEARCH METHOD
C
      CALL GOLDSEC(XFX,ND,N,U,D,SK,NS,NT,EP,0,FU,R,X,FX,
     2             IS,IT,KS,KT)
C      WRITE(6,*) ' IS=',IS,' IT=',IT,' KS=',KS,' KT=',KT
      WRITE(6,*) 'X=',(X(I),I=1,N)
      CALL GF(ND,N,X,G)
      S=GG
      GG=0.D0
         DO 50 I=1,N
   50    GG=GG+G(I)**2
      S=GG/S
         DO 60 I=1,N
   60    D(I)=G(I)+S*D(I)
   10 CONTINUE
      ITEST=2
      RETURN
      END
C-------------------------------------------------------------------
      SUBROUTINE GOLDSEC(XFX,ND,N,U,V,S,NS,NT,EP,IOPT,FU,R,X,FX,
     2                   IS,IT,KS,KT)
C
C     MINIMIZATION OF THE FUNCTION F ON THE HALF LINE
C     X=U-RV, R>=0, BY THE GOLDEN SECTION SEARCH METHOD
C
C     USES XFX SUBROUTINE
C
C     INPUT:     
C     XFX: LINE X=U-RV AND FUNCTION FX=F(X) SUBROUTINE
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     U: LINE ORIGIN VECTOR (N)
C     V: DIRECTION VECTOR (N)
C     S: SEARCH STEP SIZE
C     NS: MAXIMUM NUMBER OF ITERATIONS (STEP SEARCH)
C     NT: MAXIMUM NUMBER OF ITERATIONS (TRISECTION)
C     EP: CONVERGENCE TEST NUMBER (TRISECTION)
C     IOPT = 0, STEP SEARCH IS NOT SKIPPED
C          = 1, STEP SEARCH IS SKIPPED
C
C     OUTPUT:
C     FU:=F(U)
C     X=U+RV: APPROXIMATE SOLUTION VECTOR (N) 
C     FX:=F(X) 
C     IS = 1, STEP SEARCH TEST SATISFIED
C        = 2, STEP SEARCH TEST NOT SATISFIED
C     IT = 1, TRISECTION TEST SATISFIED
C        = 2, TRISECTION TEST NOT SATISFIED
C     KS: NUMBER OF EXECUTED ITERATIONS (STEP SEARCH)
C     KT: NUMBER OF EXECUTED ITERATIONS (TRISECTION)
C     
      DOUBLE PRECISION EP,S,U,V,X,A,B,C,D,BA,FU,FX,FA,FB,FC,FD,
     2                 P,Q,R
      DIMENSION U(ND),V(ND),X(ND)                                                                    
      IS=1
      IT=1      
      KS=0
      KT=0
C      
C     STEP SEARCH
C
      A=0.D0
      C=0.D0
      CALL XFX(ND,N,U,V,A,X,FA)
      FU=FA
      FC=FA
      B=S
      CALL XFX(ND,N,U,V,B,X,FB)
      IF(IOPT.EQ.1) GO TO 10
      IF(FA.LE.FB) GO TO 10
      DO 20 K=1,NS
      KS=K
      A=C
      FA=FC
      C=B
      FC=FB
      B=C+S
      CALL XFX(ND,N,U,V,B,X,FB)
      IF(FC.LE.FB) GO TO 10
   20 CONTINUE
      IS=2
C
C     TRISECTION
C
   10 P=(3.D0-DSQRT(5.))/2.D0
      Q=1.D0-P
      BA=B-A
      C=A+P*BA
      D=A+Q*BA
      CALL XFX(ND,N,U,V,C,X,FC)
      CALL XFX(ND,N,U,V,D,X,FD)
      DO 30 K=1,NT
      KT=K
      IF(BA.LE.EP) GO TO 40
      IF(FC.LE.FD) GO TO 50
      A=C
      BA=B-A
      C=D
      D=A+Q*BA
      FC=FD
      CALL XFX(ND,N,U,V,D,X,FD)
      GO TO 30
   50 B=D
      BA=B-A
      D=C
      C=A+P*BA
      FD=FC
      CALL XFX(ND,N,U,V,C,X,FC)
   30 CONTINUE
      IT=2
   40 R=A+BA/2.D0
      CALL XFX(ND,N,U,V,R,X,FX)
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE XFX(ND,N,U,V,R,X,FX)
      DOUBLE PRECISION U,V,R,X,FX
      DIMENSION U(ND),V(ND),X(ND)
      DO 10 I=1,N
   10 X(I)=U(I)-R*V(I)
      FX=X(1)**2+2.D0*X(2)**2+3.D0*X(3)**2
      RETURN
      END
C------------------------------------------------------------------
      SUBROUTINE GF(ND,N,X,G)
      DOUBLE PRECISION X,G
      DIMENSION X(ND),G(ND)
      G(1)=2.D0*X(1)
      G(2)=4.D0*X(2)
      G(3)=6.D0*X(3)
      RETURN
      END
