C     PROGRAM GOLDS 
C
C     GOLDEN SECTION METHOD
C
      PARAMETER(ND=10)
      DOUBLE PRECISION EP,S,U,V,X,FU,FX,R
      DIMENSION U(ND),V(ND),X(ND)
      EXTERNAL XFX
      READ(5,*) N,IOPT,NS,NT,S,EP
      READ(5,*) (U(I),I=1,N)
      READ(5,*) (V(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' IOPT=',IOPT,' NS=',NS,' NT=',NT
      WRITE(6,*) 'S=',S,' EP=',EP
      WRITE(6,*) 'U=',(U(I),I=1,N)
      WRITE(6,*) 'V=',(V(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL GOLDSEC(XFX,ND,N,U,V,S,NS,NT,EP,IOPT,FU,R,X,FX,
     2             IS,IT,KS,KT)
      WRITE(6,*) 'IS=',IS,' IT=',IT,' KS=',KS,' KT=',KT
      WRITE(6,*) 'FU=',FU
      WRITE(6,*) 'X=',(X(I),I=1,N)
      WRITE(6,*) 'FX=',FX
      STOP
      END
C------------------------------------------------------------------      
      SUBROUTINE GOLDSEC(XFX,ND,N,U,V,S,NS,NT,EP,IOPT,FU,R,X,FX,
     2                   IS,IT,KS,KT)
C
C     MINIMIZATION OF THE FUNCTION F ON THE HALF LINE
C     X=U-RV, R>=0, BY THE GOLDEN SECTION SEARCH METHOD
C
C     USES XFX SUBROUTINE
C
C     INPUT:     
C     XFX: LINE X=U-RV AND FUNCTION FX=F(X) SUBROUTINE
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     U: LINE ORIGIN VECTOR (N)
C     V: DIRECTION VECTOR (N)
C     S: SEARCH STEP SIZE
C     NS: MAXIMUM NUMBER OF ITERATIONS (STEP SEARCH)
C     NT: MAXIMUM NUMBER OF ITERATIONS (TRISECTION)
C     EP: CONVERGENCE TEST NUMBER (TRISECTION)
C     IOPT = 0, STEP SEARCH IS NOT SKIPPED
C          = 1, STEP SEARCH IS SKIPPED
C
C     OUTPUT:
C     FU:=F(U)
C     X=U-RV: APPROXIMATE SOLUTION VECTOR (N) 
C     FX:=F(X) 
C     IS = 1, STEP SEARCH TEST SATISFIED
C        = 2, STEP SEARCH TEST NOT SATISFIED
C     IT = 1, TRISECTION TEST SATISFIED
C        = 2, TRISECTION TEST NOT SATISFIED
C     KS: NUMBER OF EXECUTED ITERATIONS (STEP SEARCH)
C     KT: NUMBER OF EXECUTED ITERATIONS (TRISECTION)
C     
      DOUBLE PRECISION EP,S,U,V,X,A,B,C,D,BA,FU,FX,FA,FB,FC,FD,
     2                 P,Q,R
      DIMENSION U(ND),V(ND),X(ND)                                                                    
      IS=1
      IT=1      
      KS=0
      KT=0
C      
C     STEP SEARCH
C
      A=0.D0
      C=0.D0
      CALL XFX(ND,N,U,V,A,X,FA)
      FU=FA
      FC=FA
      B=S
      CALL XFX(ND,N,U,V,B,X,FB)
      IF(IOPT.EQ.1) GO TO 10
      IF(FA.LE.FB) GO TO 10
      DO 20 K=1,NS
      KS=K
      A=C
      FA=FC
      C=B
      FC=FB
      B=C+S
      CALL XFX(ND,N,U,V,B,X,FB)
      IF(FC.LE.FB) GO TO 10
   20 CONTINUE
      IS=2
C
C     TRISECTION
C
   10 P=(3.D0-DSQRT(5.D0))/2.D0
      Q=1.D0-P
      BA=B-A
      C=A+P*BA
      D=A+Q*BA
      CALL XFX(ND,N,U,V,C,X,FC)
      CALL XFX(ND,N,U,V,D,X,FD)
      DO 30 K=1,NT
      KT=K
      IF(BA.LE.EP) GO TO 40
      IF(FC.LE.FD) GO TO 50
      A=C
      BA=B-A
      C=D
      D=A+Q*BA
      FC=FD
      CALL XFX(ND,N,U,V,D,X,FD)
      GO TO 30
   50 B=D
      BA=B-A
      D=C
      C=A+P*BA
      FD=FC
      CALL XFX(ND,N,U,V,C,X,FC)
   30 CONTINUE
      IT=2
   40 R=A+BA/2.D0
      CALL XFX(ND,N,U,V,R,X,FX)
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE XFX(ND,N,U,V,R,X,FX)
      DOUBLE PRECISION U,V,R,X,FX
      DIMENSION U(ND),V(ND),X(ND)
      DO 10 I=1,N
   10 X(I)=U(I)-R*V(I)
      FX=X(1)**2+2.D0*X(2)**2+3.D0*X(3)**2
      RETURN
      END
