C     PROGRAM LEASQ 
C
C     LEAST SQUARES METHOD WITH GAUSS SUBROUTINE
C
      PARAMETER(MD=20,ND=5)
      DOUBLE PRECISION A,B,C,Y
      DIMENSION A(MD,ND),B(MD),C(ND,ND),Y(ND),L(ND)
      EXTERNAL DGAUSS
      READ(5,*) M,N
      READ(5,*) ((A(I,J),J=1,N),I=1,M)
      READ(5,*) (B(I),I=1,M)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'M=',M,' N=',N
      WRITE(6,*) 'A='
      DO 10 I=1,M
      WRITE(6,*) (A(I,J),J=1,N)
   10 WRITE(6,*) ' '
      WRITE(6,*) 'B=',(B(I),I=1,M)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL LEASQUA(DGAUSS,MD,ND,M,N,A,B,C,Y,IREG,L)
      WRITE(6,*) 'IREG=',IREG
      IF(IREG.EQ.0) STOP
      WRITE(6,*) 'Y=',(Y(J),J=1,N)
      STOP
      END
C-----------------------------------------------------------------
      SUBROUTINE LEASQUA(DGAUSS,MD,ND,M,N,A,B,C,Y,IREG,L)
C      
C     USES SUBROUTINE DGAUSS
C
C     ESTIMATION OF X IN THE LINEAR SYSTEM AX=B BY THE
C     LEAST SQUARES METHOD
C
C     INPUT:
C     DGAUSS: GAUSS' METHOD SUBROUTINE
C     MD,ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     M,N: DIMENSIONS OF ARRAYS  
C     A: SYSTEM MATRIX (MXN, M>=N)
C     B: RIGHT-HAND SIDE VECTOR (M)
C
C     OUTPUT:  
C     Y: ESTIMATION OF X  (N)
C         
      DOUBLE PRECISION A,B,C,Y,S,D
      DIMENSION A(MD,N),B(MD),C(ND,N),Y(ND),L(ND)
C    
C     COMPUTATION OF THE MATRIX C=(AT)A
C
      DO 10 I=1,N
         DO 10 J=1,N
         S=0.D0
            DO 20 K=1,M
   20       S=S+A(K,I)*A(K,J)
   10    C(I,J)=S
C
C     COMPUTATION OF THE VECTOR Y=(AT)B
C
      DO 30 I=1,N
      S=0.D0
         DO 40 K=1,M
   40    S=S+A(K,I)*B(K)
   30 Y(I)=S
C
C     SOLUTION OF THE SYSTEM  CX=Y  BY GAUSS'S METHOD.
C     THE SOLUTION X IS STORED IN Y
C
      CALL DGAUSS(ND,N,C,Y,0,D,IREG,L)
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE DGAUSS(ND,N,A,B,IOPT,D,IREG,L)
C       
C     DECOMPOSITION OF THE MATRIX A AND/OR SOLUTION OF THE
C     LINEAR SYSTEM AX=B BY GAUSS' ELIMINATION METHOD
C
C     INPUT:
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     A: SYSTEM MATRIX (NXN)
C     B: RIGHT-HAND SIDE VECTOR (N)
C     IOPT = 0, DECOMPOSITION OF A AND SOLUTION OF THE SYSTEM
C          = 1, DECOMPOSITION OF A ONLY
C          = 2, SOLUTION OF THE SYSTEM (SKIPS DECOMPOSITION)
C        
C     OUTPUT:
C     A: DECOMPOSED MATRIX (LOWER\UPPER PART), IF IOPT=0 OR 1
C     B:=X: SOLUTION VECTOR STORED IN B (N)
C     D: DETERMINANT OF A                                                
C     IREG = 0, SINGULAR MATRIX A
C          = 1, REGULAR MATRIX A
C     L: VECTOR OF PIVOTING INDICES (N)
C
      DOUBLE PRECISION A,B,D,S,EE
      DIMENSION A(ND,N),B(ND),L(ND)
      EE=1.D-14
      IREG=1
      NM=N-1
      IF(IOPT.EQ.2) GO TO 60
C
C     TRIANGULAR DECOMPOSITION OF A WITH MAXIMUM COLUMN PIVOTING
C     (ELIMINATION WITH MULTIPLIERS STORED IN THE LOWER PART OF A)
C   
      D=1.D0
      DO 10 K=1,NM
      KP=K+1
      M=K
         DO 20 I=KP,N
         IF(DABS(A(I,K)).GT.DABS(A(M,K))) M=I
   20    CONTINUE
      L(K)=M
      D=D*A(M,K)
      IF(DABS(A(M,K)).LE.EE) GO TO 100   
      IF(M.EQ.K) GO TO 30
      D=-D
         DO 40 J=K,N
         S=A(M,J)
         A(M,J)=A(K,J)
   40    A(K,J)=S
   30    DO 50 I=KP,N
         S=A(I,K)/A(K,K)
         A(I,K)=S
            DO 50 J=KP,N
   50       A(I,J)=A(I,J)-S*A(K,J)
   10 CONTINUE
      D=D*A(N,N)
      IF(DABS(A(N,N)).LE.EE) GO TO 100
      IF(IOPT.EQ.1) RETURN
C
C     ELIMINATION OPERATIONS ON B
C
   60 DO 70 K=1,NM
      KP=K+1
      M=L(K)
      S=B(M)
      B(M)=B(K)
      B(K)=S
         DO 70 I=KP,N
   70    B(I)=B(I)-A(I,K)*S
C      
C     BACK-SUBSTITUTION
C
      B(N)=B(N)/A(N,N)
      DO 80 I=1,NM
      K=N-I
      KP=K+1
      S=0.D0
         DO 90 J=KP,N
   90    S=S+A(K,J)*B(J)
   80 B(K)=(B(K)-S)/A(K,K)
      RETURN
  100 IREG=0
      RETURN
      END
      


      
