C     PROGRAM MARMI 
C
C     MODIFIED ARMIJO METHOD
C
      PARAMETER(ND=10)
      DOUBLE PRECISION U,G,X,EPS,SP,B,C,FU,FX,GN
      DIMENSION U(ND),G(ND),X(ND)
      EXTERNAL XFX,GF
      READ(5,*) N,SP,B,C,NIT,EPS,NS
      READ(5,*) (U(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' SP=',SP 
      WRITE(6,*) 'B=',B,' C=',C
      WRITE(6,*) 'NIT=',NIT,' EPS=',EPS,' NS=',NS 
      WRITE(6,*) 'U=',(U(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL MARMIJO(XFX,GF,ND,N,U,SP,B,C,NIT,EPS,NS,
     2             FU,X,FX,GN,G,ITEST,KIT)
      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      WRITE(6,*) 'X=',(X(I),I=1,N)
      WRITE(6,*) 'FX=',FX,' GN=',GN
      STOP
      END
C--------------------------------------------------------------------
      SUBROUTINE MARMIJO(XFX,GF,ND,N,U,SP,B,C,NIT,EPS,NS,
     2                  FU,X,FX,GN,G,ITEST,KIT)
C
C     MINIMIZATION OF THE FUNCTION F BY THE MODIFIED ARMIJO METHOD
C
C     USES XFX AND GF SUBROUTINES
C
C     INPUT:
C     XFX: LINE X=U-RV AND FUNCTION FX=F(X) SUBROUTINE
C     GF: GRADIENT OF F SUBROUTINE
C     ND: MAXIMUM DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     U: INITIAL VECTOR (N)
C     SP,B,C: STEP SEARCH PARAMETERS
C     NIT: MAXIMUM NUMBER OF ITERATIONS 
C     NS: MAXIMUM NUMBER OF STEP SEARCH ITERATIONS
C     EPS: CONVERGENCE TEST NUMBER 
C
C     OUTPUT:
C     X: LAST VECTOR ITERATE (APPROXIMATE SOLUTION) (N)
C     FX:=F(X)
C     U: BEFORE LAST VECTOR ITERATE
C     FU:=F(U)
C     GN: NORM OF (GRAD F)(X)
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     ITEST = 1, CONVERGENCE TEST SATISFIED 
C           = 2, CONVERGENCE TEST NOT SATISFIED
C
C     IS = 1: STEP SEARCH TEST SATISFIED 
C        = -2 OR 2: INSUFFICIENT LEFT OR RIGHT STEP SEARCH 
C                   ITERATIONS (NS)
C     LS: NUMBER OF EXECUTED STEP SEARCH ITERATIONS 
C     (IS AND LS ARE PRINTED ONLY, AT EACH ITERATION)
C        
      DOUBLE PRECISION U,G,X,SP,SK,A,B,C,EPS,FX,FU,GG,GN
      DIMENSION U(ND),G(ND),X(ND)
      ITEST=1
      CALL GF(ND,N,U,G)                      
      CALL XFX(ND,N,U,G,0.D0,X,FX)
      FU=FX
      SK=SP
C
      DO 10 K=0,NIT
      KIT=K
      GG=0.D0
         DO 20 I=1,N
   20    GG=GG+G(I)**2
      GN=DSQRT(GG)
      IF(GN.LE.EPS) RETURN
         DO 30 I=1,N
   30    U(I)=X(I)
      FU=FX
C      
C     STEP SEARCH
C
      IS=1
      A=SK
      CALL XFX(ND,N,U,G,A,X,FX)
      IF(FX-FU.LT.-B*A*GG) GO TO 60
         DO 40 L=1,NS
         LS=-L
         A=A*C
         CALL XFX(ND,N,U,G,A,X,FX)
         IF(FX-FU.LE.-B*A*GG) GO TO 80
   40    CONTINUE
      IS=-2
      GO TO 80
   60    DO 50 L=1,NS
         LS=L
         A=A/C
         CALL XFX(ND,N,U,G,A,X,FX)
         IF(FX-FU.GE.-B*A*GG) GO TO 70
   50    CONTINUE
      IS=2
   70 A=A*C 
      CALL XFX(ND,N,U,G,A,X,FX)
   80 CONTINUE
C      WRITE(6,*) ' IS=',IS,' LS=',LS
      WRITE(6,*) 'X=',(X(I),I=1,N)
      IF(IS.NE.1) GO TO 90
      CALL GF(ND,N,X,G)
   10 CONTINUE
   90 ITEST=2
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE XFX(ND,N,U,V,R,X,FX)
      DOUBLE PRECISION U,V,R,X,FX
      DIMENSION U(ND),V(ND),X(ND)
      DO 10 I=1,N
   10 X(I)=U(I)-R*V(I)
      FX=X(1)**2+2.D0*X(2)**2+3.D0*X(3)**2
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE GF(ND,N,X,G)
      DOUBLE PRECISION X,G
      DIMENSION X(ND),G(ND)
      G(1)=2.D0*X(1)
      G(2)=4.D0*X(2)
      G(3)=6.D0*X(3)
      RETURN
      END
