C     PROGRAM NEWTS 
C
C     NEWTON'S METHOD FOR SYSTEMS 
C
      PARAMETER(ND=10)
      DOUBLE PRECISION EPS,X,Y,A,B
      DIMENSION X(ND),Y(ND),A(ND,ND),B(ND),L(ND)
      EXTERNAL DGAUSS,F,DF
      READ(5,*) N,NIT,EPS
      READ(5,*) (X(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' NIT=',NIT,' EPS=',EPS
      WRITE(6,*) 'X=',(X(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL NEWTSYS(DGAUSS,F,DF,ND,N,NIT,EPS,X,KIT,ITEST,Y,A,B,L)
      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      IF(ITEST.EQ.0) STOP
      WRITE(6,*) 'X= ',(X(I),I=1,N)
      STOP
      END
C---------------------------------------------------------------------
      SUBROUTINE NEWTSYS(DGAUSS,F,DF,ND,N,NIT,EPS,X,KIT,ITEST,Y,A,B,L)
C     
C     SOLUTION OF THE NONLINEAR SYSTEM F(X)=0 BY NEWTON'S METHOD
C
C     USES SUBROUTINES DGAUSS,F,DF
C
C     INPUT:
C     DGAUSS: GAUSS ELIMINATION METHOD SUBROUTINE
C     F: SUBROUTINE DEFINING THE VECTOR FUNCTION F
C     DF: SUBROUTINE DEFINING THE JACOBIAN MATRIX OF F
C     ND: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     N: DIMENSION OF ARRAYS
C     NIT: MAXIMUM NUMBER OF ITERATIONS
C     EPS: CONVERGENCE TEST NUMBER
C     X: INITIAL VECTOR (N) 
C
C     OUTPUT:
C     X: APPROXIMATE SOLUTION VECTOR (N)
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     ITEST = 0, SINGULAR JACOBIAN MATRIX
C           = 1, CONVERGENCE TEST SATISFIED
C           = 2, CONVERGENCE TEST NOT SATISFIED
C        
      DOUBLE PRECISION EPS,X,Y,A,B,D,S,T
      DIMENSION X(ND),Y(ND),A(ND,N),B(ND),L(ND)
      ITEST=1
      DO 10 K=1,NIT
      KIT=K
      CALL F(ND,N,X,B)
      CALL DF(ND,N,X,A)
      CALL DGAUSS(ND,N,A,B,0,D,IREG,L)
      IF(IREG.EQ.0) GO TO 20
         DO 30 I=1,N
   30    Y(I)=X(I)
         DO 40 I=1,N
   40    X(I)=X(I)-B(I)
      WRITE(6,*) 'K=',K
      WRITE(6,*) '  X=',(X(I),I=1,N)
      S=0.D0
         DO 50 I=1,N
         T=DABS(X(I)-Y(I))
         IF(T.GT.S) S=T
   50    CONTINUE
      IF(S.LE.EPS) RETURN
   10 CONTINUE
      ITEST=2
      RETURN
   20 ITEST=0
      RETURN
      END
C-------------------------------------------------------------------      
      SUBROUTINE DGAUSS(ND,N,A,B,IOPT,D,IREG,L)
C       
C     DECOMPOSITION OF THE MATRIX A AND/OR SOLUTION OF THE
C     LINEAR SYSTEM AX=B BY GAUSS' ELIMINATION METHOD
C
C     INPUT:
C        ND: MAXIMUM COLUMNN DIMENSION OF ARRAYS
C        N: DIMENSION OF ARRAYS
C        A: SYSTEM MATRIX (NXN)
C        B: RIGHT-HAND SIDE VECTOR (N)
C        IOPT = 0, DECOMPOSITION OF A AND SOLUTION OF THE SYSTEM
C             = 1, DECOMPOSITION OF A ONLY
C             = 2, SOLUTION OF THE SYSTEM (SKIPS DECOMPOSITION)
C        
C     OUTPUT:
C        A: DECOMPOSED MATRIX (LOWER\UPPER PART), IF IOPT=0 OR 1
C        B:=X: SOLUTION VECTOR STORED IN B (N)
C        D: DETERMINANT OF A                                                
C        IREG = 0, SINGULAR MATRIX A 
C             = 1, REGULAR MATRIX A
C        L: VECTOR OF PIVOTING INDICES (N)
C
      DOUBLE PRECISION A,B,D,S,EE
      DIMENSION A(ND,N),B(ND),L(ND)
      EE=1.D-14
      IREG=1
      NM=N-1
      IF(IOPT.EQ.2) GO TO 60
C
C     TRIANGULAR DECOMPOSITION OF A WITH MAXIMUM COLUMN PIVOTING
C     (ELIMINATION WITH MULTIPLIERS STORED IN THE LOWER PART OF A)
C   
      D=1.D0
      DO 10 K=1,NM
      KP=K+1
      M=K
         DO 20 I=KP,N
         IF(DABS(A(I,K)).GT.DABS(A(M,K))) M=I
   20    CONTINUE
      L(K)=M
      D=D*A(M,K)
      IF(DABS(A(M,K)).LE.EE) GO TO 100   
      IF(M.EQ.K) GO TO 30
      D=-D
         DO 40 J=K,N
         S=A(M,J)
         A(M,J)=A(K,J)
   40    A(K,J)=S
   30    DO 50 I=KP,N
         S=A(I,K)/A(K,K)
         A(I,K)=S
            DO 50 J=KP,N
   50       A(I,J)=A(I,J)-S*A(K,J)
   10 CONTINUE
      D=D*A(N,N)
      IF(DABS(A(N,N)).LE.EE) GO TO 100
      IF(IOPT.EQ.1) RETURN
C
C     ELIMINATION OPERATIONS ON B
C
   60 DO 70 K=1,NM
      KP=K+1
      M=L(K)
      S=B(M)
      B(M)=B(K)
      B(K)=S
         DO 70 I=KP,N
   70    B(I)=B(I)-A(I,K)*S
C      
C     BACK-SUBSTITUTION
C
      B(N)=B(N)/A(N,N)
      DO 80 I=1,NM
      K=N-I
      KP=K+1
      S=0.D0
         DO 90 J=KP,N
   90    S=S+A(K,J)*B(J)
   80 B(K)=(B(K)-S)/A(K,K)
      RETURN
  100 IREG=0
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE F(ND,N,X,FX)
      DOUBLE PRECISION X,FX
      DIMENSION X(ND),FX(ND)
      FX(1)=4.D0*(X(1)-X(2))**3+2.D0*X(1)-2.D0
      FX(2)=-4.D0*(X(1)-X(2))**3+2.D0*X(2)
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE DF(ND,N,X,DFX)
      DOUBLE PRECISION X,DFX
      DIMENSION X(ND),DFX(ND,N)
      DFX(1,1)=12.D0*(X(1)-X(2))**2+2.D0
      DFX(1,2)=-12.D0*(X(1)-X(2))**2
      DFX(2,1)=DFX(1,2)
      DFX(2,2)=12.D0*(X(1)-X(2))**2+2.D0
      RETURN
      END
