C     PROGRAM NEWTO 
C
C     NEWTON'S METHOD
C
      DOUBLE PRECISION F,DF,EPS,X,FX
      EXTERNAL F,DF
      READ(5,*) M,NIT,EPS,X
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'M=',M,' NIT=',NIT,' EPS=',EPS
      WRITE(6,*) 'X=',X
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL NEWTON(F,DF,M,NIT,EPS,X,FX,KIT,ITEST)
      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      IF(ITEST.EQ.0) STOP
      WRITE(6,*) 'X=',X,' FX=',FX
      STOP
      END
C-----------------------------------------------------------------
      SUBROUTINE NEWTON(F,DF,M,NIT,EPS,X,FX,KIT,ITEST)
C         
C     SOLUTION OF THE EQUATION F(X)=0 BY NEWTON'S METHOD
C  
C     USES D.P. FUNCTIONS F AND DF 
C
C     INPUT:
C     F: FUNCTION
C     DF: DERIVATIVE OF F
C     M: ROOT MULTIPLICITY PARAMETER (NORMALLY M=1)
C     NIT: MAXIMUM NUMBER OF ITERATIONS
C     EPS: CONVERGENCE TEST NUMBER
C     X: INITIAL VALUE 
C
C     OUTPUT:
C     X: APPROXIMATE ROOT
C     FX:=F(X)
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     ITEST = 0, ZERO DERIVATIVE (DIVISION BY ZERO)
C           = 1, CONVERGENCE TEST SATISFIED
C           = 2, CONVERGENCE TEST NOT SATISFIED
C
      DOUBLE PRECISION F,DF,EPS,X,FX,DFX,Y,EE
      EXTERNAL F,DF
      EE=1.D-99
      ITEST=1
      FX=F(X)
C
      DO 10 K=1,NIT
      KIT=K
      Y=X
      DFX=DF(X)
      IF(DABS(DFX).LT.EE) GO TO 20
      X=X-DFLOAT(M)*FX/DFX
      FX=F(X)
      WRITE(6,*) 'K=',K,' X=',X,' FX=',FX
      IF(DABS(X-Y).LE.EPS) RETURN
   10 CONTINUE
      ITEST=2
      RETURN
   20 ITEST=0
      RETURN
      END
C-----------------------------------------------------------------
      DOUBLE PRECISION FUNCTION F(X)
      DOUBLE PRECISION X
      F=(X-DCOS(X))**3
      RETURN
      END
C-----------------------------------------------------------------
      DOUBLE PRECISION FUNCTION DF(X)
      DOUBLE PRECISION X
      DF=3.D0*(1.D0+DSIN(X))*(X-DCOS(X))**2
      RETURN
      END       
