C     PROGRAM PENAL 
C
C     PENALTY METHOD WITH MODIFIED ARMIJO METHOD 
C
C     INPUT:
C     PM: INITIAL PENALTY MULTIPLIER, 1<PM<10
C     EPS: INITIAL GRADIENT TEST EPSILON, 0<EPS<0.5
C     D: PENALTY MULTIPLIER PARAMETER (PM=D*PM), 1.5<D<10
C     E: GRADIENT TEST FACTOR (EPS=E*EPS), 0<E<0.5
C     NP: NUMBER OF PENALTY METHOD ITERATIONS
C
C     SEE ALSO MARMIPE SUBROUTINE BELOW
C
      PARAMETER(ND=10,MD=10)
      DOUBLE PRECISION U,G,X,PM,EPS,SP,B,C,D,E,FPU,FX,PX,FPX,GN,EM
      DIMENSION U(ND),G(ND),X(ND),EM(MD)
      EXTERNAL XFP,GFP
      READ(5,*) N,M,NP,PM,D,NIT,EPS,E,NS,SP,B,C
      READ(5,*) (U(I),I=1,N)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'N=',N,' M=',M
      WRITE(6,*) 'NP=',NP,' PM=',PM
      WRITE(6,*) 'D=',D,' NIT=',NIT
      WRITE(6,*) 'EPS=',EPS,' E=',E 
      WRITE(6,*) 'NS=',NS,' SP=',SP
      WRITE(6,*) 'B=',B,' C=',C
      WRITE(6,*) 'U=',(U(I),I=1,N)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      DO 10 I=1,N
   10 X(I)=U(I)
C
      DO 20 KP=1,NP
C      WRITE(6,*) 'KP=',KP
      PM=D*PM
      EPS=E*EPS
      SP=SP/D
      CALL MARMIPE(XFP,GFP,ND,N,MD,M,PM,U,SP,B,C,NIT,EPS,NS,
     2             FPU,X,FX,PX,FPX,GN,G,EM,ITEST,KIT)
C      WRITE(6,*) 'ITEST=',ITEST,' KIT=',KIT
      WRITE(6,*) 'X=',(X(I),I=1,N)
   20 CONTINUE
      WRITE(6,*) 'FPX=',FPX,' GN=',GN
      WRITE(6,*) 'FX=',FX,' PX=',PX
      WRITE(6,*) 'EM=',(EM(I),I=1,M)
      WRITE(6,*) 'PM=',PM,' EPS=',EPS
      WRITE(6,*) ' '
C   20 CONTINUE
      STOP
      END
C--------------------------------------------------------------------
      SUBROUTINE MARMIPE(XFP,GFP,ND,N,MD,M,PM,U,SP,B,C,NIT,EPS,NS,
     2                   FPU,X,FX,PX,FPX,GN,G,EM,ITEST,KIT)
C
C     MINIMIZATION OF THE PENALIZED FUNCTION  F(X)+PM*P(X) 
C     BY THE MODIFIED ARMIJO METHOD
C
C     USES XFP AND GFP SUBROUTINES
C
C     INPUT:
C     XFP: LINE  X=U-RV, COST FUNCTION  FX=F(X), PENALTY FUNCTION 
C          PX=P(X), AND PENALIZED FUNCTION  FPX=F(X)+PM*P(X) 
C          SUBROUTINE
C     GFP: GRAD(F+PM*P) SUBROUTINE
C     ND,MD: MAXIMUM DIMENSIONS OF ARRAYS
C     N,M: DIMENSIONS OF ARRAYS
C     U: INITIAL VECTOR (N)
C     PM: PENALTY MULTIPLIER
C     SP,B,C: STEP SEARCH PARAMETERS
C     NIT: MAXIMUM NUMBER OF ITERATIONS 
C     NS: MAXIMUM NUMBER OF STEP SEARCH ITERATIONS
C     EPS: CONVERGENCE TEST NUMBER 
C
C     OUTPUT:
C     X: LAST VECTOR ITERATE (APPROXIMATE SOLUTION) (N)
C     FX=F(X): COST 
C     PX=P(X): PENALTY 
C     FPX=F(X)+PM*P(X): PENALIZED COST 
C     EM: VECTOR OF EXTREMALITY MULTIPLIERS (M)
C     U: BEFORE LAST VECTOR ITERATE
C     FPU:=F(U)+PM*P(U)
C     GN: NORM OF GRAD(F+PM*P) AT X
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     ITEST = 1, CONVERGENCE TEST SATISFIED 
C           = 2, CONVERGENCE TEST NOT SATISFIED
C
C     IS = 1: STEP SEARCH TEST SATISFIED 
C        = -2 OR 2: INSUFFICIENT LEFT OR RIGHT STEP SEARCH 
C                   ITERATIONS (NS)
C     LS: NUMBER OF EXECUTED STEP SEARCH ITERATIONS 
C     (IS AND LS ARE PRINTED ONLY, AT EACH ITERATION)
C        
      DOUBLE PRECISION U,G,X,PM,SP,SK,A,B,C,EPS,FPU,FX,PX,FPX,
     2                 GG,GN,EM
      DIMENSION U(ND),G(ND),X(ND),EM(MD)
      ITEST=1
      CALL GFP(ND,N,MD,M,PM,X,G,EM)
      CALL XFP(ND,N,PM,X,G,0.D0,X,FX,PX,FPX)
      SK=SP
C
      DO 10 K=0,NIT
      KIT=K
      GG=0.D0
         DO 20 I=1,N
   20    GG=GG+G(I)**2
      GN=DSQRT(GG)
      IF(GN.LE.EPS) RETURN
         DO 30 I=1,N
   30    U(I)=X(I)
      FPU=FPX
C      
C     STEP SEARCH
C
      IS=1
      A=SK
      CALL XFP(ND,N,PM,U,G,A,X,FX,PX,FPX)
      IF(FPX-FPU.LT.-B*A*GG) GO TO 60
         DO 40 L=1,NS
         LS=-L
         A=A*C
         CALL XFP(ND,N,PM,U,G,A,X,FX,PX,FPX)
         IF(FPX-FPU.LE.-B*A*GG) GO TO 80
   40    CONTINUE
      IS=-2
      GO TO 80
   60    DO 50 L=1,NS
         LS=L
         A=A/C
         CALL XFP(ND,N,PM,U,G,A,X,FX,PX,FPX)
         IF(FPX-FPU.GE.-B*A*GG) GO TO 70
   50    CONTINUE
      IS=2
   70 A=A*C 
      CALL XFP(ND,N,PM,U,G,A,X,FX,PX,FPX)
C      WRITE(6,*) 'IS=',IS,'  LS=',LS
   80 CONTINUE   
      IF(IS.NE.1) GO TO 90
      CALL GFP(ND,N,MD,M,PM,X,G,EM)
   10 CONTINUE
   90 ITEST=2
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE XFP(ND,N,PM,U,V,R,X,FX,PX,FPX)
      DOUBLE PRECISION PM,U,V,R,X,FX,PX,FPX
      DIMENSION U(ND),V(ND),X(ND)
      DO 10 I=1,N
   10 X(I)=U(I)-R*V(I)
      FX=0.5D0*(X(1)**2+(X(2)-1.D0)**2+X(3)**2)
      PX=0.5D0*((DMAX1(0.D0,1.D0-X(1)))**2
     2         +(DMAX1(0.D0,1.5D0-0.5D0*X(1)-X(3)))**2
     3         +(DMAX1(0.D0,X(1)+X(2)+X(3)-9.D0))**2
     4         +(0.5D0*(X(1)-2.D0)**2+0.5D0*X(2)**2-2.5D0)**2)
      FPX=FX+PM*PX
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE GFP(ND,N,MD,M,PM,X,G,EM)
      DOUBLE PRECISION PM,X,G,EM
      DIMENSION X(ND),G(ND),EM(MD)
      EM(1)=PM*(DMAX1(0.D0,1.D0-X(1)))
      EM(2)=PM*(DMAX1(0.D0,1.5D0-0.5D0*X(1)-X(3)))
      EM(3)=PM*(DMAX1(0.D0,X(1)+X(2)-X(3)-9.D0))
      EM(4)=PM*(0.5D0*(X(1)-2.D0)**2+0.5D0*X(2)**2-2.5D0)
      G(1)=X(1)-EM(1)-0.5D0*EM(2)+EM(3)+(X(1)-2.D0)*EM(4)
      G(2)=X(2)-1.D0+EM(3)+X(2)*EM(4)
      G(3)=X(3)-EM(2)+EM(3)
      RETURN
      END
