C     PROGRAM PREDC 
C
C     PREDICTOR-CORRECTOR METHOD
C
      DOUBLE PRECISION A,B,H,X,Y,Z 
      DIMENSION Z(4)
      EXTERNAL RUNKUT4,F
      READ(5,*) A,B,N,NC,Y
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'A=',A,' B=',B
      WRITE(6,*) 'N=',N,' NC=',NC
      WRITE(6,*) 'Y=',Y
      H=(B-A)/DFLOAT(N)
      X=A
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      WRITE(6,*) 'X=',X,'  Y=',Y
      DO 10 K=1,N
      CALL PREDCOR(RUNKUT4,F,K,X,Y,Z,H,NC)
   10 WRITE(6,*) 'X=',X,'  Y=',Y
      STOP
      END
C-----------------------------------------------------------------
      SUBROUTINE PREDCOR(RUNKUT4,F,K,X,Y,Z,H,NC)
C
C     PREDICTOR-CORRECTOR METHOD (ONE STEP K)
C
C     USES SUBROUTINE RUNKUT4 AND D.P. FUNCTION F
C
C     INPUT:
C     RUNKUT4: FOURTH ORDER RUNGE-KUTTA METHOD SUBROUTINE
C     F: FUNCTION
C     K: STEP INDEX
C     X: INITIAL VARIABLE VALUE
C     Y: INITIAL VALUE
C     H: STEP SIZE
C     NC: NUMBER OF CORRECTOR ITERATIONS
C   
C     OUTPUT: 
C     X:=X+H
C     Y: APPROXIMATE SOLUTION AT X
C
      DOUBLE PRECISION F,H,H2,H3,X,Y,Z
      DIMENSION Z(4)
      EXTERNAL F
      H2=H+H
      H3=H2+H
C
C     COMPUTATION OF STARTING VALUES BY THE FOURTH ORDER 
C     RUNGE-KUTTA METHOD
C     
      IF(K.EQ.1) Z(1)=Y
      IF(K.GT.3) GO TO 10
      CALL RUNKUT4(F,X,Y,H)
      Z(K+1)=Y
      RETURN
C
C     PREDICTOR
C
   10 Y=Z(4)+(55.D0*F(X,Z(4))-59.D0*F(X-H,Z(3))
     2        +37.D0*F(X-H2,Z(2))-9.D0*F(X-H3,Z(1)))*H/24.D0
C      
C     CORRECTOR     
C
      X=X+H
      DO 20 L=1,NC
   20 Y=Z(4)+(9.D0*F(X,Y)+19.D0*F(X-H,Z(4))
     2        -5.D0*F(X-H2,Z(3))+F(X-H3,Z(2)))*H/24.D0
C
C     NEW VALUES Z(I)       
C
      DO 30 I=1,3
   30 Z(I)=Z(I+1)
      Z(4)=Y
      RETURN
      END
C-----------------------------------------------------------------
      SUBROUTINE RUNKUT4(F,X,Y,H)
C           
C     FOURTH ORDER RUNGE-KUTTA METHOD (ONE STEP)
C
C     USES FUNCTION F
C
C     INPUT:
C     F: FUNCTION
C     X: INITIAL VARIABLE VALUE
C     Y: INITIAL VALUE
C     H: STEP SIZE
C
C     OUTPUT:
C     X:=X+H
C     Y: APPROXIMATE SOLUTION AT X
C
      DOUBLE PRECISION F,H,X,X2,Y,Z1,Z2,Z3,Z4
      EXTERNAL F
      Z1=H*F(X,Y)
      X2=X+H/2.D0
      Z2=H*F(X2,Y+Z1/2.D0)
      Z3=H*F(X2,Y+Z2/2.D0)
      X=X+H
      Z4=H*F(X,Y+Z3)
      Y=Y+(Z1+2.D0*Z2+2.D0*Z3+Z4)/6.D0
      RETURN
      END
C-----------------------------------------------------------------
      DOUBLE PRECISION FUNCTION F(X,Y)
      DOUBLE PRECISION X,Y
      F=DSIN(X+Y)
      RETURN
      END
