C     PROGRAM SIMPL 
C
C     SIMPLEX METHOD
C
      PARAMETER(MD=5,ND=10,NMD=15)
      DOUBLE PRECISION A,B,C,X,F
      DIMENSION A(MD,ND),B(MD),C(NMD),L(NMD),X(NMD)
      READ(5,*) M,N,N1,N2,NIT,IBF
      READ(5,*) ((A(I,J),J=1,N),I=1,M)
      READ(5,*) (B(I),I=1,M)
      READ(5,*) (C(J),J=1,N1)
      WRITE(6,*) 'INPUT:'
      WRITE(6,*) 'M=',M,' N=  ',N
      WRITE(6,*) 'N1=',N1,' N2= ',N2
      WRITE(6,*) 'NIT=',NIT,' IBF=',IBF
      WRITE(6,*) 'A='
      DO 10 I=1,M
      WRITE(6,*) (A(I,J),J=1,N)
   10 WRITE(6,*) ' '
      WRITE(6,*) 'B=',(B(I),I=1,M)
      WRITE(6,*) 'C=',(C(J),J=1,N1)
      WRITE(6,*) ' '
      WRITE(6,*) 'OUTPUT:'
      CALL SIMPLEX(MD,ND,NMD,M,N,N1,N2,A,B,C,L,NIT,IBF,X,F,KIT,IT)
      IF(IBF.EQ.0) GO TO 20
      WRITE(6,*) '*PHASE I*'
      WRITE(6,*) 'IT=',IT,' KIT=',KIT 
      WRITE(6,*) 'F=',F
      IF(F.GT.1.D-10) STOP
      CALL SIMPLEX(MD,ND,NMD,M,N,N1,N2,A,B,C,L,NIT,2,X,F,KIT,IT)
   20 WRITE(6,*) '*PHASE II*'
      WRITE(6,*) 'IT=',IT,' KIT=',KIT
      WRITE(6,*) 'F=',F
      IF(IT.NE.1) STOP
      NM=N+M
      DO 30 J=1,NM
   30 WRITE(6,*) 'J=',J,'  X=',X(J),'  L=',L(J)
      STOP
      END
C----------------------------------------------------------------      
      SUBROUTINE SIMPLEX(MD,ND,NMD,M,N,N1,N2,A,B,C,L,NIT,IBF,X,F,
     2                   KIT,IT)
C
C     SOLUTION OF AN ORIGINAL LINEAR PROGRAMMING PROBLEM WHICH 
C     HAS BEEN TRANSFORMED IN STANDARD FORM:
C        MINIMIZE F(X)=(CT)X, SUBJECT TO [A|I]X=B>=0 AND X>=0 
C     BY DANTZIG'S SIMPLEX METHOD      
C
C               ............................................   .
C               .                        .                 .   .
C               .                        .                 .   .
C               .                        .                 .   .
C               .                        .        I        .   .
C  SYSTEM:    M .           A            .    IDENTITY     .   . B
C               .                        .     MATRIX      .   .
C               .                        .                 .   .
C               .                        .                 .   .
C               ............................................   .                                      
C               1                 N1    N      N2         N+M
C
C  VARIABLES:   [-------NONBASIC--------]------BASIC-------]
C               [-----ORIGINAL-----]---SLACK----]ARTIFICIAL] 
C         
C  COST COEF: C [******************]-----0------]----1-----]                                
C                                            
C     INPUT:                     
C     MD,ND,NMD: MAXIMUM COLUMN DIMENSION OF ARRAYS
C     M,N: DIMENSIONS OF ARRAYS
C     N1,N2: INDEX BOUNDS OF VARIABLES.        
C            IF IBF=1 OR 2, X(N2+1),...,X(N+M) ARE ARTIFICIAL
C     A: SYSTEM MATRIX (NONBASIC PART)  (MXN)
C     B: RIGHT-HAND SIDE VECTOR (M)
C     C: VECTOR OF COST COEFFICIENTS (N+M), IF IBF=0 OR 1.
C        C(J), FOR J<=N1, NEED ONLY BE GIVEN 
C     NIT: MAXIMUM NUMBER OF ITERATIONS
C     IBF = 0, PHASE II ONLY IS EXECUTED (FEASIBLE ORIGINAL PROBLEM)
C         = 1, PHASE I IS EXECUTED (NONFEASIBLE ORIGINAL PROBLEM)
C         = 2, PHASE II IS EXECUTED (AFTER PHASE I)
C     X: INITIAL VECTOR (N+M), IF IBF=2 (COMPUTED IN PHASE I) 
C     L: INITIAL PIVOTING PERMUTATION VECTOR (N+M), IF IBF=2 
C        (COMPUTED IN PHASE I)
C
C     OUTPUT:
C     X: SOLUTION VECTOR, IF IT=1   
C     F: MINIMUM COST (ARTIFICIAL IF IBF=1, ORIGINAL IF IBF=0 OR 2)
C     L: PERMUTATION VECTOR (N+M)
C     C: VECTOR OF PERMUTED COST COEFFICIENTS (N+M)
C     KIT: NUMBER OF EXECUTED ITERATIONS
C     IT = 0, THE PROBLEM HAS NO SOLUTION
C        = 1, THE PROBLEM HAS A SOLUTION
C        = 2, INSUFFICIENT ITERATIONS (NIT)
C       
      DOUBLE PRECISION A,B,C,X,F,D,S,R
      DIMENSION A(MD,N),B(MD),C(NMD),L(NMD),X(NMD)
      INTEGER P,Q
C                 
C     INITIAL VALUES OF C,X,L,F
C     
      NM=N+M
      N1P=N1+1
      N2P=N2+1
      IF(IBF.EQ.2) GO TO 23 
      DO 30 J=N1P,N2
   30 C(J)=0.D0
      IF(IBF.NE.1) GO TO 21
      DO 31 J=N2P,NM
   31 C(J)=1.D0
   21 DO 32 J=1,N
   32 X(J)=0.D0
      DO 33 I=1,M
   33 X(N+I)=B(I)    
      DO 34 J=1,NM
   34 L(J)=J
      IF(IBF.NE.0) GO TO 22
      F=0.D0
      GO TO 10    
   22 NN=N2P-N
      F=0.D0
      DO 35 I=NN,M
   35 F=F+B(I)
      GO TO 10 
   23 F=0.D0
      DO 36 I=1,M
      NI=N+I
      IF(L(NI).LE.N1) F=F+C(NI)*B(I)      
   36 CONTINUE   
C   
C     ITERATIONS
C
   10 DO 200 K=0,NIT 
      KIT=K
C
C     SEARCH OF PIVOT COLUMN INDEX Q
C     
      D=0.D0
      Q=0
         DO 50 J=1,N
         IF(IBF.EQ.2.AND.L(J).GT.N2) GO TO 50
         S=C(J)
         IF(IBF.NE.1) GO TO 55
         IF(L(J).LE.N2) S=0.D0
   55       DO 60 I=1,M
            NI=N+I
            IF(IBF.NE.1) GO TO 66
            IF(L(NI).LE.N2) GO TO 60
            GO TO 65
   66       IF(L(NI).GT.N2) GO TO 60
   65       S=S-C(NI)*A(I,J)
   60       CONTINUE
         IF(S.GE.0.D0) GO TO 50
         IF(S.GE.D) GO TO 50
         D=S
         Q=J
   50    CONTINUE
      IF(Q.EQ.0) GO TO 140
C
C     SEARCH OF PIVOT ROW INDEX P 
C
      R=1.D+30
      P=0
         DO 70 I=1,M
         IF(IBF.EQ.2.AND.L(N+I).GT.N2) GO TO 70
         IF(A(I,Q).LE.0.D0) GO TO 70
         S=B(I)/A(I,Q)
         IF(S.GE.R) GO TO 70
         R=S
         P=I
   70    CONTINUE
      IF(P.EQ.0) GO TO 150
C
C     PERMUTATION OF L(Q) AND L(N+P), C(Q) AND C(N+P)  
C
      NP=N+P
      J=L(Q)
      L(Q)=L(NP)
      L(NP)=J
      S=C(Q)
      C(Q)=C(NP)
      C(NP)=S
C            
C     PIVOTING OPERATIONS       
C
         DO 100 J=1,N
         IF(IBF.EQ.2.AND.L(J).GT.N2) GO TO 100
         IF(J.EQ.Q) GO TO 100
         A(P,J)=A(P,J)/A(P,Q)
  100    CONTINUE
      A(P,Q)=1.D0/A(P,Q)
      B(P)=R
C     
         DO 80 I=1,M   
         IF(I.EQ.P) GO TO 80
            DO 90 J=1,N
            IF(IBF.EQ.2.AND.L(J).GT.N2) GO TO 90
            IF(J.EQ.Q) GO TO 90
            A(I,J)=A(I,J)-A(I,Q)*A(P,J)
   90       CONTINUE
         B(I)=B(I)-A(I,Q)*B(P)
         A(I,Q)=-A(I,Q)*A(P,Q)
   80    CONTINUE
C
C     NEW COST F
C
      F=F+R*D
C
  200 CONTINUE
      GO TO 160
C      
C     SOLUTION VECTOR X
C
  140 DO 120 J=1,N
  120 X(L(J))=0.D0
      DO 125 I=1,M
  125 X(L(N+I))=B(I)
      IT=1
      RETURN
C
  150 IT=0
      RETURN
  160 IT=2
      RETURN
      END


